@@ -5,24 +5,22 @@ Created on Thu Sep 27 08:28:20 2021
 
 @author: daniel
 """
+from pyBIA.data_processing import crop_image, concat_channels
 from tensorflow.keras.preprocessing.image import ImageDataGenerator
 from scipy.ndimage import rotate
 import matplotlib.pyplot as plt
+from warnings import warn
 import numpy as np
 import random
 
-from pyBIA.data_processing import crop_image, concat_channels
-from warnings import warn
-
-
 def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5, height_shift=5, 
-    horizontal=True, vertical=True, rotation=360, fill='nearest', image_size=50):
+    horizontal=True, vertical=True, rotation=True, fill='nearest', image_size=50, mask_size=None, num_masks=None):
     """
     This function takes in one image and applies data augmentation techniques.
     Shifts and rotations occur at random, for example, if width_shift is set
     to 10, then an image shift between -10 and 10 pixels will be chosen from a 
     random uniform distribution. Rotation angle is also chosen from a random uniform 
-    distribution, between zero and the rotation argument. 
+    distribution, between zero and 360. 
 
     Note:
         This function is used for offline data augmentation! In practice,
@@ -46,31 +44,45 @@ def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5
             If set to zero no vertical shifts will be performed. Defaults to 5 pixels.
         horizontal (bool): If False no horizontal flips are allowed. Defaults to True.
         vertical (bool): If False no vertical reflections are allowed. Defaults to True.
-        rotation (int): The rotation angle in degrees. Defaults to 360 for full rotation allowed.
+        rotation (int): If True full 360 rotation is allowed, if False no rotation is performed.
+            Defaults to True.
         fill (str): This is the treatment for data outside the boundaries after roration
             and shifts. Default is set to 'nearest' which repeats the closest pixel values.
             Can set to: {"constant", "nearest", "reflect", "wrap"}.
         image_size (int, bool): The length/width of the cropped image. This can be used to remove
-            anomalies caused by the fill. Defaults to 50, the pyBIA standard. This can also
+            anomalies caused by the fill (defaults to 50). This can also
             be set to None in which case the image in its original size is returned.
 
-    Note:
-        The training set pyBIA uses includes augmented images. The original image size was
-        100x100 pixels before augmentation, these were cropped to 50x50 after the augmentation
-        procedure so as to remove rotational effects at the outer boundaries of the image.
-        This cropping is controlled via the image_size parameter.
-
     Returns:
         Array containing the augmented images. When input, channel2 and channel3 yield 
         additionl outputs, respectively.
     """
 
-    if isinstance(width_shift, int) == False or isinstance(height_shift, int) == False or isinstance(rotation, int) == False:
+    if isinstance(width_shift, int) == False or isinstance(height_shift, int) == False:
         raise ValueError("Shift parameters must be integers indicating +- pixel range")
+    if mask_size is not None:
+        if num_masks is None:
+            raise ValueError('Need to input num_masks parameter.')
+    if num_masks is not None:
+        if mask_size is None:
+            raise ValueError('Need to input mask_size parameter.')
+
+    if rotation:
+        rotation = 360
+    else:
+        rotation = 0
 
     def image_rotation(data):
-        return rotate(data, np.random.choice(range(rotation+1), 1)[0], reshape=False, order=1, prefilter=False)
-    
+        """
+        Function for the image data genereation which hardcodes the rotation parameter of the parent function.
+        The order parameter to 0 to ensure that the rotation is performed using nearest-neighbor interpolation, 
+        which minimizes the amount of distortion introduced into the image. Additionally, the reshape parameter is False, 
+        which will ensure that the rotated image has the same shape as the original image. The prefilter parameter is also 
+        set to False to prevent any additional processing from being performed on the image prior to rotation.
+        """
+        return rotate(data, np.random.choice(range(rotation+1), 1)[0], reshape=False, order=0, prefilter=False)
+        
+    #Tensorflow Image Data Generator with shifts and flips. While fill is also an option, in practice it is best to crop an oversized image 
     datagen = ImageDataGenerator(
         width_shift_range=width_shift,
         height_shift_range=height_shift,
@@ -78,6 +90,7 @@ def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5
         vertical_flip=vertical,
         fill_mode=fill)
 
+    #The rotation child function is only added to the Image Data Generator if rotation parameter is input
     if rotation != 0:
         datagen.preprocessing_function = image_rotation
 
@@ -90,19 +103,19 @@ def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5
         else:
             raise ValueError("Input data must be 2D for single sample or 3D for multiple samples")
 
-    augmented_data = []
-    seeds = []
+    augmented_data, seeds = [], []
     for i in np.arange(0, len(data)):
         original_data = data[i].reshape((1,) + data[-i].shape)
         for j in range(batch):
             seed = int(random.sample(range(int(1e6)), 1)[0])
             seeds.append(seed)
             augment = datagen.flow(original_data, batch_size=1, seed=seed)
-            augmented_data.append(augment[0][0])
+            if mask_size is not None:
+                augmented_data.append(random_cutout(augment[0][0], mask_size=mask_size, num_masks=num_masks, seed=seed, augmenting=True))
+            else:
+                augmented_data.append(augment[0][0])
 
-    augmented_data = np.array(augmented_data)
-    if augmented_data is not None:
-        augmented_data = resize(augmented_data, size=image_size)
+    augmented_data = resize(np.array(augmented_data), size=image_size)
 
     if channel2 is None:
         return augmented_data
@@ -121,12 +134,13 @@ def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5
             original_data = data[i].reshape((1,) + data[-i].shape)
             for j in range(batch):
                 augment = datagen.flow(original_data, batch_size=1, seed=seeds[k])
-                augmented_data2.append(augment[0][0])
+                if mask_size is not None:
+                    augmented_data2.append(random_cutout(augment[0][0], mask_size=mask_size, num_masks=num_masks, seed=seeds[k], augmenting=True))
+                else:
+                    augmented_data2.append(augment[0][0])
                 k+=1
 
-        augmented_data2 = np.array(augmented_data2)
-        if augmented_data2 is not None:
-            augmented_data2 = resize(augmented_data2, size=image_size)
+        augmented_data2 = resize(np.array(augmented_data2), size=image_size)
 
     if channel3 is None:
         return augmented_data, augmented_data2
@@ -144,12 +158,13 @@ def augmentation(channel1, channel2=None, channel3=None, batch=10, width_shift=5
             original_data = data[i].reshape((1,) + data[-i].shape)
             for j in range(batch):
                 augment = datagen.flow(original_data, batch_size=1, seed=seeds[k])
-                augmented_data3.append(augment[0][0])
+                if mask_size is not None:
+                    augmented_data3.append(random_cutout(augment[0][0], mask_size=mask_size, num_masks=num_masks, seed=seeds[k], augmenting=True))
+                else:
+                    augmented_data3.append(augment[0][0])
                 k+=1
 
-    augmented_data3 = np.array(augmented_data3)
-    if augmented_data3 is not None:
-        augmented_data3 = resize(augmented_data3, size=image_size)            
+    augmented_data3 = resize(np.array(augmented_data3), size=image_size)
 
     return augmented_data, augmented_data2, augmented_data3
 
@@ -160,12 +175,10 @@ def resize(data, size=50):
 
     Args:
         data (array): 2D array
-        size (int): length/width of the output array. Defaults to
-            50 pixels which is the pyBIA convention.
+        size (int): length/width of the output array. Defaults to 50 pixels.
 
     Returns:
         The cropped out array
-
     """
 
     if len(data.shape) == 3 or len(data.shape) == 4:
@@ -180,10 +193,8 @@ def resize(data, size=50):
     if width != height:
         raise ValueError("Can only resize square images")
     if width == size:
-        warn("No resizing necessary, image shape is already in desired size", stacklevel=2)
-        if len(data.shape) == 4:
-            data = data[:, :, :, 0]
-        return data
+        #print("No resizing necessary, image shape is already in desired size, returning original data...")
+        return data 
 
     if len(data.shape) == 2:
         resized_data = crop_image(np.array(np.expand_dims(data, axis=-1))[:, :, 0], int(width/2.), int(height/2.), size)
@@ -219,6 +230,46 @@ def resize(data, size=50):
 
     return resized_data
 
+def random_cutout(images, mask_size=16, num_masks=1, seed=None, augmenting=False):
+    """
+    Applies the cutout data augmentation technique to a sample of 2D images.
+    This method applies `num_masks` random positioned (mask_size x mask_size) black squares to each image.
+
+    Args:
+        data (numpy array): A 3D array of shape (num_images, height, width).
+        mask_size (int): The size of the cutout mask. Defaults to 16.
+        num_masks (int): Number of masks to apply to each image. Defaults to 1.
+        seed (int): Seed for the random number generator. Defaults to None.
+        augmenting (bool): If False the image shape is assumed to be (num_img, height, width), if True
+            it is assumed to be reversed. Defaults to False.
+
+    Returns:
+        A 3D array of the same shape as data, with cutout applied.
+    """
+
+    #images = copy.deepcopy(data)
+
+    if augmenting: #The data augmentation procedure requires the input to be inversed.
+        width, height, num_images = images.shape
+    else:
+        num_images, height, width = images.shape
+
+    if seed is not None:
+        np.random.seed(seed)
+
+    for i in range(num_images):
+        for j in range(num_masks):
+            if height - mask_size > 0:
+                h = np.random.randint(height - mask_size)
+                w = np.random.randint(width - mask_size)
+                images[i, h:h+mask_size, w:w+mask_size] = 0
+            else:
+                raise ValueError('WARNING: Mask size is too large for the image input!')
+
+    np.random.seed(1909) #Set back to pyBIA default
+
+    return images
+
 def plot(data, cmap='gray', title=''):
     """
     Plots 2D array using a robust colorbar range to
@@ -231,7 +282,6 @@ def plot(data, cmap='gray', title=''):
 
     Returns:
         AxesImage.
-        
     """
     
     index = np.where(np.isfinite(data))
